import * as utils from '../src/utils'

describe('Utils', () => {
  const data = {
    name: 'John',
    username: undefined,
    email: 'john@example.com',
    groups: [{ name: 'admin', permissions: 'rwx' }],
    tasks: [],
  }

  describe('get', () => {
    it('should return value of a property with given key or path', () => {
      expect(utils.get(data, 'email')).toBe(data.email)
      expect(utils.get(data, 'groups[0].name')).toBe(data.groups[0].name)
    })
  })

  describe('isEmpty', () => {
    it('should return true if given value is an empty array or object', () => {
      expect(utils.isEmpty(data)).not.toBe(true)
      expect(utils.isEmpty({})).toBe(true)
      expect(utils.isEmpty([])).toBe(true)
    })

    it('should return false if given value is an object with properties', () => {
      expect(utils.isEmpty(data)).toBe(false)
      expect(utils.isEmpty(['data'])).toBe(false)
    })

    it('should return true if given value has no own enumerable properties', () => {
      expect(utils.isEmpty('data')).not.toBe(true)
      expect(utils.isEmpty(1)).toBe(true)
      expect(utils.isEmpty(true)).toBe(true)
      expect(utils.isEmpty(false)).toBe(true)
    })
  })


  describe('getNonEmptyKeys', () => {
    it('should get keys of properties which value is not empty', () => {
      expect(utils.getNonEmptyKeys(data)).toEqual(['name', 'email', 'groups'])
    })
  })

  describe('getChangedKeys', () => {
    it('should get keys of properties which value changed', () => {
      const mutatedData = {
        ...data,
        groups: [...data.groups, { name: 'guest', permissions: 'r' }],
      }
      expect(utils.getChangedKeys(data, mutatedData)).toEqual(['groups'])
      expect(utils.getChangedKeys(data, data)).toEqual([])
    })
  })
})
