import isEmpty from 'lodash/isEmpty'
import get from 'lodash/get'

/**
 * A reduce function for an object
 *
 * @param {Object} obj
 * @param {function} func
 * @param {*} [initialValue = {}]
 * @returns {*}
 */
const reduceObject = (obj, func, initialValue = {}) => (
  Object.entries(obj).reduce(func, initialValue)
)

/**
 * Gets changed properties keys
 *
 * @param {Object} original
 * @param {Object} mutated
 * @returns {Array}
 * */
const getChangedKeys = (original, mutated) => (
  reduceObject(mutated, (keys, [key, value]) => (
    value === original[key] ? keys : [...keys, key]
  ), [])
)

/**
 * Gets keys of non empty properties
 *
 * @param {Object} obj
 * @returns {Array}
 */
const getNonEmptyKeys = obj => (
  reduceObject(obj, (keys, [key, value]) => (
    isEmpty(value) ? keys : [...keys, key]
  ), [])
)

export {
  isEmpty,
  get,
  getChangedKeys,
  reduceObject,
  getNonEmptyKeys,
}
